@@ -329,7 +329,7 @@ userMatchesString(char *buf, char *user)
 }
 
 /*
- * user:passwod[:user:password]
+ * user:password[:user:password]
  */
 PasswordMapping *
 pool_get_user_credentials(char *username)
@@ -452,7 +452,7 @@ pool_reopen_passwd_file(void)
 
 /*
  * function first uses the password in the argument, if the argument is empty
- * string or NULL, it looks for the password for user in pool_passwd file.
+ * string or NULL, it looks for the password for uset in pool_passwd file.
  * The returned password is always in plain text or md5 hashed and palloc'd
  * (if not null)
  */
@@ -464,7 +464,7 @@ get_pgpool_config_user_password(char *username, char *password_in_config)
 	PasswordMapping *password_mapping = NULL;
 
 	/*
-	 * if the password specified in config is empty strin or NULL look for the
+	 * if the password specified in config is empty string or NULL look for the
 	 * password in pool_passwd file
 	 */
 	if (password_in_config == NULL || strlen(password_in_config) == 0)
@@ -690,9 +690,10 @@ read_pool_key(char *key_file_path)
  * -2: password is not found
  */
 int
-check_password_type_is_not_md5(char *username, char *password_in_config)
+chceck_password_type_is_not_md5(char *username, char *password_in_config)
 {
 	PasswordType passwordType = PASSWORD_TYPE_UNKNOWN;
+	PasswordMapping *password_mapping = NULL;
 
 	/*
 	 * if the password specified in config is empty string or NULL look for the
@@ -700,14 +701,12 @@ check_password_type_is_not_md5(char *username, char *password_in_config)
 	 */
 	if (password_in_config == NULL || strlen(password_in_config) == 0)
 	{
-		PasswordMapping *password_mapping = NULL;
 		password_mapping = pool_get_user_credentials(username);
 		if (password_mapping == NULL)
 		{
 			return -2;
 		}
 		passwordType = password_mapping->pgpoolUser.passwordType;
-		delete_passwordMapping(password_mapping);
 	}
 	else
 	{
