@@ -8,8 +8,7 @@
 # -p installation path of Postgres
 # -j JDBC driver path
 # -m install (install pgpool-II and use that for tests) / noinstall : Default install
-# -s unix socket directory
-# -c test pgpool using sample scripts and config files
+# -s unix socket directory 
 # -d start pgpool with debug option
 
 dir=`pwd`
@@ -17,7 +16,7 @@ MODE=install
 PG_INSTALL_DIR=/usr/local/pgsql/bin
 PGPOOL_PATH=/usr/local
 JDBC_DRIVER=/usr/local/pgsql/share/postgresql-9.2-1003.jdbc4.jar
-export USE_REPLICATION_SLOT=true
+#export USE_REPLICATION_SLOT=true
 export log=$dir/log
 fail=0
 ok=0
@@ -93,12 +92,7 @@ function export_env_vars
  	fi
 	
 	echo "using pgpool-II at "$PGPOOL_PATH
-
-	export PGPOOL_VERSION=`$PGPOOL_PATH/bin/pgpool --version 2>&1`
-
 	export PGPOOL_INSTALL_DIR=$PGPOOL_PATH
-	# where to look for pgpool.conf.sample files.
-	export PGPOOLDIR=${PGPOOLDIR:-"$PGPOOL_INSTALL_DIR/etc"}
 
 	PGPOOLLIB=${PGPOOL_INSTALL_DIR}/lib
 	if [ -z "$LD_LIBRARY_PATH" ];then
@@ -122,8 +116,7 @@ function print_info
 	echo ${CBLUE}"*************************"${CNORM}
 
 	echo "REGRESSION MODE          : "${CBLUE}$MODE${CNORM}
-	echo "Pgpool-II version        : "${CBLUE}$PGPOOL_VERSION${CNORM}
-	echo "Pgpool-II install path   : "${CBLUE}$PGPOOL_PATH${CNORM}
+	echo "PGPOOL-II                : "${CBLUE}$PGPOOL_PATH${CNORM}
 	echo "PostgreSQL bin           : "${CBLUE}$PGBIN${CNORM}
 	echo "PostgreSQL Major version : "${CBLUE}$PGVERSION${CNORM}
 	echo "pgbench                  : "${CBLUE}$PGBENCH_PATH${CNORM}
@@ -143,7 +136,6 @@ function print_usage
 	printf "  -j   FILE                Postgres jdbc jar file path\n" >&2
 	printf "  -s   DIRECTORY           unix socket directory\n" >&2
 	printf "  -t   TIMEOUT             timeout value for each test (sec)\n" >&2
-	printf "  -c                       test pgpool using sample scripts and config files\n" >&2
 	printf "  -d                       start pgpool with debug option\n" >&2
 	printf "  -?                       print this help and then exit\n\n" >&2
 	printf "Please read the README for details on adding new tests\n" >&2
@@ -152,7 +144,7 @@ function print_usage
 
 trap "echo ; exit 0" SIGINT SIGQUIT
 
-while getopts "p:m:i:j:b:s:t:cd?" OPTION
+while getopts "p:m:i:j:b:s:t:d?" OPTION
 do
   case $OPTION in
     p)  PG_INSTALL_DIR="$OPTARG";;
@@ -162,7 +154,6 @@ do
     b)  PGBENCH_PATH="$OPTARG";;
     s)  PGSOCKET_DIR="$OPTARG";;
     t)  TIMEOUT="$OPTARG";;
-    c)  export TEST_SAMPLES="true";;
     d)  export PGPOOLDEBUG="true";;
     ?)  print_usage
         exit 2;;
@@ -205,25 +196,10 @@ fi
 for i in $dirs
 do
 	cd $i
-
-	# skip the test if there's no test.sh
-	if [ ! -f test.sh ];then
-		cd ..
-		continue;
-	fi
-
 	echo -n "testing $i..."
 	clean_all
 	timeout $TIMEOUT ./test.sh > $log/$i 2>&1
 	rtn=$?
-
-	check_segfault
-	if [ $? -eq 0 ];then
-		echo "fail: Segmentation fault detected" >> $log/$i
-		segfault=1
-		rtn=1
-	fi
-
 	if [ $rtn = 0 ];then
 		echo ${CGREEN}"ok."${CNORM}
 		ok=`expr $ok + 1`
@@ -231,11 +207,7 @@ do
 		echo "timeout."
 		timeout=`expr $timeout + 1`
 	else
-		if [ "$segfault" = "1" ];then
-			echo ${CRED}"segfault."${CNORM}
-		else
-			echo ${CRED}"failed."${CNORM}
-		fi
+		echo ${CRED}"failed."${CNORM}
 		fail=`expr $fail + 1`
 	fi
 
@@ -243,6 +215,6 @@ do
 
 done
 
-total=`expr $ok + $fail + $timeout`
+total=`expr $ok + $fail`
 
 echo "out of $total ok:$ok failed:$fail timeout:$timeout"
